//--------------------------------------------------------------------------------------------------
//
// Title       : InstMemory	(ALU Test)
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Instruction memory that is preloaded with a program to test
//					some basic ALU and memory instructions
//
//-------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps
module InstMemory(
	input	[31:0]	address,
	output	[31:0]	Inst
);

// This is the actual memory array 
reg [31:0] memory[0:255];


// op-codes for our CPU
parameter ALU		= 6'h00; // op =  0
parameter LW		= 6'h23; // op = 35
parameter SW		= 6'h2b; // op = 43
parameter BEQ		= 6'h04; // op =  4 
parameter ADDI		= 6'h08; // op =  8
parameter J			= 6'h02; // op =  2
parameter HALT		= 6'h3f; // op = 63
parameter LUI		= 6'h10; // op = 16
parameter ORI		= 6'h0d; // op = 13
parameter JAL		= 6'h03; // op =  3

// functions for ALU operations
parameter ADD 		= 6'h20; // funct = 32
parameter SUB 		= 6'h22; // funct = 34
parameter AND 		= 6'h24; // funct = 36
parameter OR 		= 6'h25; // funct = 37
parameter SLT 		= 6'h2a; // funct = 42
parameter JR		= 6'h08; // funct =  8

parameter shftX 	= 5'hxx; // we won't be using shift amount

// register aliases used in MIPS assembly
parameter rz 		= 5'h00; // register 0 stores a zero
parameter v0 		= 5'h02; // used for function result
parameter v1		= 5'h03; // "                      "
parameter a0      	= 5'h04; // argument 0 for calls
parameter a1		= 5'h05;
parameter a2		= 5'h06;
parameter a3      	= 5'h07; 
parameter t0      	= 5'h08; // temps 0-7 
parameter t1		= 5'h09; // by convention these aren't saved
parameter t2		= 5'h0A; // across procedure calls
parameter t3 		= 5'h0B;
parameter t4		= 5'h0C;
parameter t5		= 5'h0D;
parameter t6		= 5'h0E;
parameter t7		= 5'h0F;
parameter s0      	= 5'h10; // saved 0-7
parameter s1		= 5'h11; // by convention any code modifying these registers
parameter s2		= 5'h12; // must save the current values and restore the values
parameter s3      	= 5'h13; // before returning from the function
parameter s4		= 5'h14;  
parameter s5		= 5'h15; 
parameter s6		= 5'h16;
parameter s7		= 5'h17;
parameter sp     	= 5'h1D; // stack pointer
parameter fp      	= 5'h1E; // frame pointer
parameter ra      	= 5'h1F; // return address



integer i;
initial begin
	memory[8'h00] = {ADDI,rz ,t0 ,      16'h0007};			// t0 = 7
	memory[8'h01] = {ADDI,rz ,a0 ,      16'hffff};			// Set address of i/o write port	(a0 = 0xffff)
	memory[8'h02] = {LUI ,rz ,a1 ,      16'h0004}; 			// Set i/o command to debug output	(a1 = 0x0004_0000)
	memory[8'h03] = {ALU ,t0 ,a1 ,a1 ,shftX,  OR};			// Use t0 as i/o value				(a1 = a1 | t0)
	memory[8'h04] = {SW  ,a0 ,a1 ,      16'h0000};			// Perform i/o						(*a0 = a1)

	for(i = 8'h05; i <= 8'hff; i = i + 1) begin
		memory[i] = 32'hxzxzxzxz;  // fill rest of memory with invalid stuff
	end
end

// data is always read
assign Inst = memory[address[7:0]];

endmodule
