//--------------------------------------------------------------------------------------------------
//
// Title       : PCupdate
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Select the source for the next instruction
//					Source is selected by PCSel input, has to be set by controller
//
// 0. PC = Next							(Move to next Instruction)
// 1. PC = Next + offset				(Conditional Branch)
// 2. PC = Reg							(Jump to Register value)
// 3. PC = {Next[31:26],Jump Target}	(J instruction; short jump)
//
//-------------------------------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module PCupdate (
	input	[31:0]	Reg,		// Register with target address
	input	[31:0]	Next,		// Value of incremented PC
	input	[25:0]	Inst,		// Low bits of instruction word
	input	[1:0]	PCSel,		// Which input to select
	output	[31:0]	PC			// Output: address for next instruction
);


reg [31:0] src;

// Aliases:

// BEQ (and others) set PC to a value relative to the next PC. To allow
// for loops, need ability for a negative offset, so sign extend the
// offset provided as bits 15:0 of the instruction
wire [31:0] offset		= {{16{Inst[15]}},Inst};	// Sign extend the immediate...
wire [31:0] Branch		= Next + offset;			// ...and calculate target

wire [31:0] Jump		= {Next[31:26],Inst[25:0]}; // Used by J instruction

  
// Set new program counter according to PCSel input
//
assign PC = (PCSel[1])
				? ((PCSel[0]) ? Jump : Reg)
				: ((PCSel[0]) ? Branch : Next);

endmodule
